'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var PropTypes = require('prop-types');
require('core-js/modules/web.dom-collections.iterator.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var PropTypes__default = /*#__PURE__*/_interopDefaultLegacy(PropTypes);

const treeService = {
  /**
   * 
   * @param {Object} dataObject 
   * @returns 
   */
  mapToTree: dataObject => {
    const treeNodes = treeService.mapToTreeNodes(dataObject);
    return treeService.mapToParentAndChild(treeNodes);
  },

  /**
   * 
   * @param {Array} nodes 
   */
  mapToParentAndChild: nodes => {
    return nodes.map(node => {
      if (node.hasParent) {
        const parentNode = nodes.find(element => element.name === node.parent);
        parentNode.hasChildren = true;
        parentNode.childNodes.push(node);
        node.parentNode = parentNode;
      }

      return node;
    });
  },

  /**
   * @params (Object) data
   * @returns (treeNode)
   */
  mapToTreeNode: dataObject => {
    var _dataObject$parent;

    return {
      name: dataObject === null || dataObject === void 0 ? void 0 : dataObject.name,
      hasParent: ((_dataObject$parent = dataObject.parent) === null || _dataObject$parent === void 0 ? void 0 : _dataObject$parent.length) > 0,
      parent: dataObject === null || dataObject === void 0 ? void 0 : dataObject.parent,
      open: false,
      checked: false,
      childNodes: [],
      state: 'closed',
      context: dataObject === null || dataObject === void 0 ? void 0 : dataObject.context
    };
  },

  /**
   * 
   * @param {Array} data 
   */
  mapToTreeNodes: data => {
    return data.map(element => {
      return treeService.mapToTreeNode(element);
    });
  },

  /**
   * 
   * @param {Object} node 
   * @param {Boolean} selectParents
   * @param {Boolean} selectChildren
   */
  changeNode: (node, selectParents, selectChildren) => {
    node.checked = !node.checked;

    if (node.hasParent && selectParents) {
      treeService.modifyParent(node, selectParents);
    }

    if (node.hasChildren && selectChildren) {
      treeService.modifyChildren(node, selectChildren);
    }
  },

  /**
   * 
   * @param {Object} node 
   * @param {Boolean} selectChildren 
   */
  modifyChildren: (node, selectChildren) => {
    node.childNodes.map(childNode => {
      if (node.checked !== childNode.checked && selectChildren) {
        childNode.checked = !childNode.checked;
      }

      if (childNode.hasChildren && selectChildren) {
        treeService.modifyChildren(childNode, selectChildren);
      }
    });
  },

  /**
   * 
   * @param {Object} node  
   */
  modifyParent: node => {
    while (node.parentNode) {
      let parentNode = node.parentNode;

      if (!parentNode.checked && node.checked) {
        parentNode.checked = !parentNode.checked;
      }

      node = parentNode;
    }
  },

  /**
   * 
   * @param {Object} node 
   */
  toggleNode: node => {
    node.open = !node.open;
  }
};

const serviceContext = /*#__PURE__*/React.createContext(treeService);

const useServiceContext = () => {
  return React.useContext(serviceContext);
};

const treeContext = /*#__PURE__*/React.createContext();

const Provider = props => {
  const service = useServiceContext();
  const [tree, _setTree] = React.useState([]);
  React.useEffect(() => {
    _setTree(service.mapToTree(props.data));
  }, []);
  const treeContainer = {
    tree: tree,
    setTree: value => {
      _setTree(value);
    }
  };
  return /*#__PURE__*/React__default["default"].createElement(serviceContext.Provider, {
    value: service
  }, /*#__PURE__*/React__default["default"].createElement(treeContext.Provider, {
    value: treeContainer
  }, props.children));
};

Provider.propTypes = {
  children: PropTypes__default["default"].node,
  data: PropTypes__default["default"].array
};

var e=[],t=[];function n(n,r){if(n&&"undefined"!=typeof document){var a,s=!0===r.prepend?"prepend":"append",d=!0===r.singleTag,i="string"==typeof r.container?document.querySelector(r.container):document.getElementsByTagName("head")[0];if(d){var u=e.indexOf(i);-1===u&&(u=e.push(i)-1,t[u]={}),a=t[u]&&t[u][s]?t[u][s]:t[u][s]=c();}else a=c();65279===n.charCodeAt(0)&&(n=n.substring(1)),a.styleSheet?a.styleSheet.cssText+=n:a.appendChild(document.createTextNode(n));}function c(){var e=document.createElement("style");if(e.setAttribute("type","text/css"),r.attributes)for(var t=Object.keys(r.attributes),n=0;n<t.length;n++)e.setAttribute(t[n],r.attributes[t[n]]);var a="prepend"===s?"afterbegin":"beforeend";return i.insertAdjacentElement(a,e),e}}

var css$1 = ".branch{\n    list-style: none;\n}\n\n.branch  details{\n    display: inline-block;\n}\n.branch summary, input{\n    cursor: pointer;\n}\n.branch ul{\n    padding-left: 30px;\n}";
n(css$1,{});

var css = ".leaf{\n    list-style: none;\n}\n\n.leaf input{\n    margin-left: 18px;\n}";
n(css,{});

const useTreeContext = () => {
  return React.useContext(treeContext);
};

const Leaf = props => {
  let treeNode = props.node;
  const checkbox = props.checkboxes;
  const service = useServiceContext();
  const tree = useTreeContext().tree;
  const setTree = useTreeContext().setTree;

  const handleChange = () => {
    service.changeNode(treeNode, props.selectParents, props.selectChildren);
    const newTree = [...tree];
    const index = newTree.indexOf(treeNode);
    newTree[index] = treeNode;
    setTree(newTree);

    if (props.onSelect) {
      props.onSelect();
    }
  };

  return /*#__PURE__*/React__default["default"].createElement("li", {
    className: "leaf"
  }, checkbox && /*#__PURE__*/React__default["default"].createElement("input", {
    type: "checkbox",
    role: "checkbox",
    onChange: handleChange,
    checked: treeNode.checked
  }), treeNode === null || treeNode === void 0 ? void 0 : treeNode.name);
};

Leaf.propTypes = {
  checkboxes: PropTypes__default["default"].bool,
  node: PropTypes__default["default"].object,
  onSelect: PropTypes__default["default"].func,
  selectChildren: PropTypes__default["default"].bool,
  selectParents: PropTypes__default["default"].bool
};

const Branch = props => {
  const nodes = props.nodes;
  const checkboxes = props.checkboxes;
  const service = useServiceContext();
  const tree = useTreeContext().tree;
  const setTree = useTreeContext().setTree;
  const branches = nodes.map((node, index) => {
    if (node.hasChildren) {
      const handleClick = () => {
        treeService.toggleNode(node);
        const newTree = [...tree];
        const index = newTree.indexOf(node);
        newTree[index] = node;
        setTree(newTree);
      };

      const handleChange = () => {
        service.changeNode(node, props.selectParents, props.selectChildren);
        const newTree = [...tree];
        const index = newTree.indexOf(node);
        newTree[index] = node;
        setTree(newTree);

        if (props.onSelect) {
          props.onSelect();
        }
      };

      return /*#__PURE__*/React__default["default"].createElement("li", {
        className: "branch",
        key: index
      }, /*#__PURE__*/React__default["default"].createElement("details", null, /*#__PURE__*/React__default["default"].createElement("summary", {
        onClick: handleClick,
        role: "tree-list"
      }, checkboxes && /*#__PURE__*/React__default["default"].createElement("input", {
        type: "checkbox",
        onChange: handleChange,
        checked: node.checked
      }), node.name), /*#__PURE__*/React__default["default"].createElement("ul", null, /*#__PURE__*/React__default["default"].createElement(Branch, {
        nodes: node.childNodes,
        checkboxes: checkboxes,
        onSelect: props.onSelect,
        selectParents: props.selectParents,
        selectChildren: props.selectChildren
      }))));
    } else {
      return /*#__PURE__*/React__default["default"].createElement(Leaf, {
        node: node,
        checkboxes: checkboxes,
        onSelect: props.onSelect,
        selectParents: props.selectParents,
        selectChildren: props.selectChildren
      });
    }
  });
  return /*#__PURE__*/React__default["default"].createElement(React__default["default"].Fragment, null, branches);
};

Branch.propTypes = {
  checkboxes: PropTypes__default["default"].bool,
  nodes: PropTypes__default["default"].array,
  onSelect: PropTypes__default["default"].func,
  selectChildren: PropTypes__default["default"].bool,
  selectParents: PropTypes__default["default"].bool
};

const Index = props => {
  const tree = useTreeContext().tree;
  const checkboxes = props.checkboxes;
  const leafs = tree.filter(treeNode => treeNode.hasParent === false);
  return /*#__PURE__*/React__default["default"].createElement("ul", null, /*#__PURE__*/React__default["default"].createElement(Branch, {
    nodes: leafs,
    checkboxes: checkboxes,
    onSelect: props.onSelect,
    selectParents: props.selectParents,
    selectChildren: props.selectChildren
  }));
};

Index.propTypes = {
  checkboxes: PropTypes__default["default"].bool,
  children: PropTypes__default["default"].node,
  onSelect: PropTypes__default["default"].func,
  selectChildren: PropTypes__default["default"].bool,
  selectParents: PropTypes__default["default"].bool
};

/**
 * @todo use summary for collapsible elements!
 * @param {*} props 
 * @returns 
 */

const Tree = props => {
  return /*#__PURE__*/React__default["default"].createElement(Provider, {
    data: props.data
  }, /*#__PURE__*/React__default["default"].createElement(Index, {
    checkboxes: props.checkboxes,
    onSelect: props.onSelect,
    selectParents: props.selectParents,
    selectChildren: props.selectChildren
  }));
};

Tree.propTypes = {
  checkboxes: PropTypes__default["default"].bool,
  data: PropTypes__default["default"].array,
  onSelect: PropTypes__default["default"].func,
  selectChildren: PropTypes__default["default"].bool,
  selectParents: PropTypes__default["default"].bool
};
Tree.defaultProps = {
  checkboxes: true,
  selectChildren: true,
  selectParents: true
};

exports.Tree = Tree;
